unit hiAdjustControls;

interface

uses Windows,Kol,Share,Debug,Win;

const
  StrParam = 'left'      + #13#10 +
             'top'       + #13#10 +
             'width'     + #13#10 +
             'height'    + #13#10 +
             'align'     + #13#10 +
             'color'     + #13#10 +
             'text'      + #13#10 +
             'caption'   + #13#10 +
             'visible'   + #13#10 +
             'enabled'   + #13#10 +
             'checked'   + #13#10 +
             'selected'  + #13#10 +
             'select'    + #13#10 +
             'fontname'  + #13#10 +
             'fontcolor' + #13#10 +
             'fontsize'  + #13#10 +
             'fontstyle' + #13#10 +
             'fontcharset';  

  ColorValues: array [0..46] of TColor = (
    clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,clGray,
    clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite,
    clMoneyGreen,clSkyBlue,clCream,clMedGray,clActiveBorder,clActiveCaption,
    clAppWorkSpace,clBackground,clBtnFace,clBtnHighlight,clBtnShadow,    
    clBtnText,clCaptionText,clDefault,clGrayText,clHighlight,clHighlightText,
    clInactiveBorder,clInactiveCaption,clInactiveCaptionText,clInfoBk,
    clInfoText,clMenu,clMenuText,clNone,clScrollBar,cl3DDkShadow,
    cl3DLight,clWindow,clWindowFrame,clWindowText );

  ColorNames: array [0..46] of String = (
    'black','maroon','green','olive','navy','purple','teal','gray',
    'silver','red','lime','yellow','blue','fuchsia','aqua','white',
    'moneygreen','skyblue','cream','medgray','activeborder','activecaption',
    'appworkspace','background','btnface','btnhighlight','btnshadow',    
    'btntext','captiontext','default','graytext','highlight','highlighttext',
    'inactiveborder','inactivecaption','inactivecaptiontext','infobk',
    'infotext','menu','menutext','none','scrollbar','3ddkshadow',
    '3dlight','window','windowframe','windowtext' );

type
  THIAdjustControls = class(TDebug)
   private
    FListSet,
    FListExcept,
    FListPar: PStrList;
    procedure SetParameters(const Value: string);
    procedure SetExceptList(const Value: string);
    function GetIntColor(const Value: string): integer;    
    function GetStrColor(const Value: integer): string;
   public
    _data_NameControl,
    _data_Parameters,
    _data_ExceptList,
    _event_onSet,
    _event_onEnumNameControls,
    _event_onEndEnumNameControls,
    _event_onEnumParameters,
    _event_onEndEnumParameters: THI_Event;
    
    constructor Create;
    destructor Destroy; override;
    procedure _work_doSet(var _Data: TData; Index: word);
    procedure _work_doEnumNameControls(var _Data: TData; Index: word);
    procedure _work_doEnumParameters(var _Data: TData; Index: word);
    
    property _prop_Parameters: string write SetParameters;
    property _prop_ExceptList: string write SetExceptList;
  end;
    
implementation

uses hiStr_Enum;

constructor THIAdjustControls.Create;
begin
   inherited Create;
   FListSet := NewStrList;
   FListPar := NewStrList;
   FListExcept := NewStrList; 
   FListPar.SetText(StrParam, false);
end;

destructor THIAdjustControls.Destroy;
begin
   FListSet.Free;
   FListPar.Free;
   FListExcept.Free;
   inherited;
end;

function THIAdjustControls.GetIntColor;
var
  i: integer;
begin
  for i := 0 to 46 do
    if Value = ColorNames[i] then
    begin
      Result := ColorValues[i];
      exit;
    end;
  Result := str2int(Value);
end;

function THIAdjustControls.GetStrColor;
var
  i: integer;
begin
  for i := 0 to 46 do
    if Color2RGB(Value) = Color2RGB(ColorValues[i]) then
    begin
      Result := ColorNames[i];
      exit;
    end;
  Result := int2str(Value);
end;

procedure THIAdjustControls.SetParameters;
begin
  FListSet.SetText(Value, false);
end;

procedure THIAdjustControls.SetExceptList;
begin
  FListExcept.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls._work_doSet;
var
  tmpControl: PControl;
  i, j, val: integer;
  str, s_name, s_prop, exc: string;

   procedure change(ind: integer);
   var
     k: integer;
   begin
     with tmpControl{$ifndef F_P}^{$endif} do
     case ind of
       0: Left := str2int(str);
       1: Top := str2int(str);
       2: Width := str2int(str);
       3: Height := str2int(str);
       4: begin
            if (str ='none')  or (str ='0') then Align := caNone   else
            if (str ='left')  or (str ='1') then Align := caLeft   else
            if (str ='top')   or (str ='2') then Align := caTop    else         
            if (str ='rigth') or (str ='3') then Align := caRight  else 
            if (str ='bottom')or (str ='4') then Align := caBottom else 
            if (str ='client')or (str ='5') then Align := caClient; 
            Invalidate;
          end;
       5: Color := GetIntColor(LowerCase(str)); 
     6,7: begin
            Replace(str, '|', #13#10);
            if isForm then
              SetWindowText(Handle, @str[1])
            else
              Text := str; // text = caption
          end;  
       8: Visible := boolean(ord(str2int(str)));
       9: Enabled := boolean(ord(str2int(str)));
   10,11: Checked := boolean(ord(str2int(str)));
      12: begin // select -  
            val := str2int(str);
            if (val < 0) or (val >= Count) then val := -1;
            CurIndex := val;
          end;
      13: Font.FontName := str;
      14: Font.Color := GetIntColor(LowerCase(str));
      15: Font.FontHeight := _hi_SizeFnt(str2int(str));
      16: begin
            Font.FontStyle := [];
            for k := 1 to length(str) do
            begin
              if str[k] = 'b' then Font.FontStyle := Font.FontStyle + [fsBold]
              else if str[k] = 'i' then Font.FontStyle := Font.FontStyle + [fsItalic]
              else if str[k] = 'u' then Font.FontStyle := Font.FontStyle + [fsUnderline]
              else if str[k] = 's' then Font.FontStyle := Font.FontStyle + [fsStrikeOut];
            end;
          end;
      17: Font.FontCharset := str2int(str);    
     end;
   end;

begin
  str := ReadString(_Data, _data_Parameters);
  if str <> '' then FListSet.SetText(str, false);
  exc := ReadString(_Data, _data_ExceptList);
  if exc <> '' then FListExcept.SetText(LowerCase(exc), false);
  for i := 0 to ListOfNameControls.Count - 1 do
  begin
    tmpControl := PControl(ListOfNameControls.Objects[i]);
    with tmpControl{$ifndef F_P}^{$endif}  do
    if isControl or isForm then
    begin
      for j := 0 to FListSet.Count - 1 do
      begin
        str := FListSet.Items[j];
        if str <> '' then
        begin
          s_name := LowerCase(fparse(str,'='));
          if ((s_name = LowerCase(ListOfNameControls.Items[i])) or (s_name = 'all')) and
             (FListExcept.IndexOf(s_name) = -1) then
          begin
           s_prop := LowerCase(fparse(str,'='));
           change(FListPar.IndexOf(s_prop));
          end;
        end;
      end;
    end;
  end;
  _hi_OnEvent(_event_onSet);
end;

procedure THIAdjustControls._work_doEnumNameControls;
var
  i: integer;
  exc: string;
begin
  exc := ReadString(_Data, _data_ExceptList);
  if exc <> '' then FListExcept.SetText(LowerCase(exc), false);
  for i := 0 to ListOfNameControls.Count - 1 do
    if FListExcept.IndexOf(ListOfNameControls.Items[i]) = -1 then 
      _hi_onEvent(_event_onEnumNameControls, ListOfNameControls.Items[i]);
  _hi_onEvent(_event_onEndEnumNameControls);      
end;

procedure THIAdjustControls._work_doEnumParameters;
var
  r: real;
  sFontSize: integer;
  tmpControl: PControl;
  name, sFontStyle, exc, sText: string;
  dtname, dtprop, dtval: TData;
  
  procedure outparameters(prop, value: string);
  begin
    dtString(dtprop, prop);
    dtString(dtval, value);
    dtval.ldata := @dtprop; 
    _hi_onEvent_(_event_onEnumParameters, dtname);
  end;

begin
  name := LowerCase(ReadString(_Data, _data_NameControl));
  if name = '' then exit;
  exc := ReadString(_Data, _data_ExceptList);
  if exc <> '' then FListExcept.SetText(LowerCase(exc), false);
  if FListExcept.IndexOf(name) <> -1 then exit;    
  tmpControl := PControl(ListOfNameControls.Objects[ListOfNameControls.IndexOf(name)]);
  if not(tmpControl.isControl or tmpControl.isForm) then exit;
  dtString(dtname, name); 
  dtname.ldata := @dtval;
  with tmpControl{$ifndef F_P}^{$endif} do
  begin
    outparameters('left', int2str(Left));
    outparameters('top', int2str(Top));
    outparameters('width', int2str(Width));
    outparameters('height', int2str(Height));
    case ord(Align) of
      0: outparameters('align', 'none');
      1: outparameters('align', 'left');
      2: outparameters('align', 'top');         
      3: outparameters('align', 'rigth'); 
      4: outparameters('align', 'bottom'); 
      5: outparameters('align', 'client'); 
    end;            
    outparameters('color', GetStrColor(Color));
    sText := Text;
    Replace(sText, #13#10, '|');
    outparameters('text', sText);
    outparameters('caption', sText);
    outparameters('visible', int2str(ord(Visible)));
    outparameters('enabled', int2str(ord(Enabled)));
    outparameters('checked', int2str(ord(Checked)));                    
    outparameters('selected', int2str(ord(Checked)));
    outparameters('select', int2str(CurIndex));    
    outparameters('fontname', Font.FontName);  
    outparameters('fontcolor', GetStrColor(Font.Color));
    r := ((Font.FontHeight * -72) - 36) / ScreenDPI;
    sFontSize := Trunc(r);
    if Frac(r) > 0 then Inc(sFontSize);
    outparameters('fontsize', int2str(sFontSize));
    sFontStyle := '';
    if fsBold in Font.FontStyle then  
      sFontStyle := sFontStyle + 'b';
    if fsItalic in Font.FontStyle then
      sFontStyle := sFontStyle + 'i';     
    if fsUnderline in Font.FontStyle then
      sFontStyle := sFontStyle + 'u';     
    if fsStrikeOut in Font.FontStyle then
      sFontStyle := sFontStyle + 's';     
    outparameters('fontstyle', sFontStyle);     
    outparameters('fontcharset', int2str(Font.FontCharset));
  end;
  _hi_onEvent(_event_onEndEnumParameters);   
end;

end.